package net.javajigi.user.service;

import java.util.List;

import net.javajigi.audit.AuditObject;
import net.javajigi.security.Permission;
import net.javajigi.security.PermissionType;
import net.javajigi.user.ExceedAdminUserException;
import net.javajigi.user.ExistedUserException;
import net.javajigi.user.PasswordMismatchException;
import net.javajigi.user.UserNotFoundException;
import net.javajigi.user.dao.UserDao;
import net.javajigi.user.model.User;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

public abstract class UserServiceImpl implements UserService {
	private final Log logger = LogFactory.getLog(getClass());

	static final String ADMIN_USER = "Administrator";
	static final String NORMAL_USER = "User";
	static final String CREATE = "add";
	static final String DELETE = "delete";

	private UserDao userDao;

	public abstract AuditObject getAudit();
	
	public void setUserDao(UserDao userDao) {
		this.userDao = userDao;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.javajigi.user.spring.UserService#create(net.javajigi.user.model.User)
	 */
	@Transactional(propagation = Propagation.REQUIRED, rollbackFor = ExceedAdminUserException.class)
	public int addUser(User user) throws ExistedUserException, ExceedAdminUserException {
		AuditObject audit = getAudit();
		if (user.isAdmin()) {
			audit.create(ADMIN_USER + " : " + user.getUserId(), CREATE);
		} else {
			audit.create(NORMAL_USER + " : " + user.getUserId(), CREATE);
		}

		user.setUserDao(userDao);
		
		int result = user.create();

		if (logger.isDebugEnabled()) {
			logger.debug("Added " + user.getUserId() + " in User System!!");
		}

		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.javajigi.user.spring.UserService#update(net.javajigi.user.model.User)
	 */
	public int updateUser(User user) {
		int result = userDao.update(user);

		if (logger.isDebugEnabled()) {
			logger.debug("Updated " + user.getUserId() + " in User System!!");
		}

		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.javajigi.user.spring.UserService#remove(java.lang.String)
	 */
	@Permission(value = PermissionType.ADMINISTRATOR)
	public int removeUser(String userId) {
		int result = userDao.delete(userId);

		if (logger.isDebugEnabled()) {
			logger.debug("Removed " + userId + " in User System!!");
		}

		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.javajigi.user.spring.UserService#findUser(java.lang.String)
	 */
	public User findUser(String userId) throws UserNotFoundException {
		User user = userDao.findUser(userId);

		if (user == null) {
			throw new UserNotFoundException(userId);
		}

		if (logger.isDebugEnabled()) {
			logger.debug(userId + " : " + user);
		}
		return user;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.javajigi.user.spring.UserService#findUserList()
	 */
	public List<User> findUserList() {
		List<User> userList = userDao.findUserList();

		if (logger.isDebugEnabled()) {
			if (userList != null) {
				logger.debug("User Size : " + userList.size());
			} else {
				logger.debug("User Size : 0");
			}
		}

		return userList;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.javajigi.user.spring.UserService#login(java.lang.String, java.lang.String)
	 */
	public boolean login(String userId, String password) throws PasswordMismatchException, UserNotFoundException {
		User user = findUser(userId);

		if (!user.isMatchPassword(password)) {
			throw new PasswordMismatchException(userId);
		}

		return true;
	}
}
